/****************************************************************************/
/*                       Beizer approximci megvalositsa                  */
/****************************************************************************/

import java.awt.*;
import java.applet.*;
import java.lang.Math;

public class Bezier extends Applet
{
   Frame f;
   BezierControls controls;
   BezierCanvas c = new BezierCanvas();

   public void init()
   {
      setBackground(Color.green);
      f = new MyWindow("Bezier curve ( , )");
      f.setLayout(new BorderLayout());
      f.setBackground(Color.black);
      f.add("Center", c);
      f.add("South", controls = new BezierControls(c,f));
      f.resize(500,300);
      f.show();
   }

   public void start() {
      controls.enable();
   }

   public void stop() {
      controls.disable();
   }

   public boolean handleEvent(Event e)
   {
      if (e.id == Event.WINDOW_DESTROY) {
         System.exit(0);
      }
      return false;
   }

   public static void main(String args[])
   {
      Bezier bezier = new Bezier();

      bezier.init();
      bezier.start();
   }

   public void paint (Graphics g)
   {
      g.drawRect(2,2,size().width-6,size().height-6);
      g.drawString("Bezier should be running in a window now...",20,33);
   }
}

class BezierCanvas extends Canvas
{
   final int   CROSS   = 3;
   final int   EPSILON = 2*CROSS;
   final int   POINTS  = 0;
   final int   LINES   = 1;
   final int   CURVE   = 2;
   final int   CLEAR   = 3;
   final int   OPEN    = 4;
   final int   CLOSE   = 5;
   final int   SHOW    = 6;
   final int   HIDE    = 7;
   final int   MOVE    = 8;
   final int   PUT     = 9;
   final int   DRAW    = 10;
   final int   UNDRAW  = 11;
   final int   N       = 30;
   final int   MAX     = 4;
   final float Wx1     = 6f;
   final float Wy1     = 0f;
   final float Wx2     = 0f;
   final float Wy2     = 6f;

   int x,y, pos=0;
   boolean clear=false;
   int open_or_close=OPEN, move_or_put=PUT;
   int show_or_hide_lines=HIDE, draw_or_undraw=DRAW;
   int which_point_to_move;
   int MODE;
   int Vx1, Vy1=10, Vx2=10, Vy2;

   float tomb_x[] = new float[MAX+3];
   float tomb_y[] = new float[MAX+3];

   public boolean mouseDown (Event e, int x, int y)
   {
      boolean handled = false;
      int i, distance;
      switch(move_or_put)
      {
         case PUT:
            if (!(e.x<Vx1 && e.x>Vx2 && e.y>Vy1 && e.y<Vy2)) break;
            if (++pos>MAX) { pos=MAX; break; }
            tomb_x[pos]=v2w_x(x);
            tomb_y[pos]=v2w_y(y);
            repaint();
            handled=true;
            break;
         case MOVE:
            for (i=1; i<=pos; ++i)
            {
               distance = (int)Math.sqrt(Math.pow(x-w2v_x(tomb_x[i]),2)+Math.pow(y-w2v_y(tomb_y[i]),2));
               if (distance<=EPSILON) { which_point_to_move = i; return true; }
            }
            which_point_to_move=0;
            break;
      }//end of switch
      return handled;
   }

   public boolean mouseDrag (Event e, int x, int y)
   {
      if (which_point_to_move==0 || move_or_put==PUT) return false;
      if (x<Vx1 && x>Vx2 && y>Vy1 && y<Vy2)
      {
         tomb_x[which_point_to_move] = v2w_x(x);
         tomb_y[which_point_to_move] = v2w_y(y);
         repaint();
         return true;
      }
      return false;
   }

   public void paint (Graphics g)
   {
      g.setColor(Color.white);
      g.drawRect(Vx2,Vy1,Vx1=size().width-20,Vy2=size().height-20);
      if (clear) { clear=false; return; }
      if (show_or_hide_lines==SHOW && pos>=4) show_lines(g);
      light_points(g);
      if (draw_or_undraw==DRAW && pos>=4) curve_fitting(g);
   }

   public void light_points(Graphics g)
   {
      g.setColor(Color.yellow);
      for (int i=1; i<=pos; ++i)
      {
         int x=w2v_x(tomb_x[i]), y=w2v_y(tomb_y[i]);
         g.drawOval(x-CROSS,y-CROSS, 2*CROSS,2*CROSS);
//         g.drawLine(x-CROSS,y+CROSS, x+CROSS,y-CROSS);
      }
   }

   public void curve_fitting(Graphics g)
   {
      g.setColor(Color.white);
      int i,j, from, to;
      boolean first;
      float X, Y, X_prev=0, Y_prev=0, t, xA, xB, xC, xD, yA, yB, yC, yD;
      float a0, a1, a2, a3, b0, b1, b2, b3;

      tomb_x[0]=tomb_x[pos]; tomb_y[0]=tomb_y[pos];
      tomb_x[pos+1]=tomb_x[1]; tomb_y[pos+1]=tomb_y[1];
      tomb_x[pos+2]=tomb_x[2]; tomb_y[pos+2]=tomb_y[2];

      if (open_or_close==CLOSE) { from=1; to=pos;   }
      else                      { from=2; to=pos-2; }
      for (i=from; i<=to; ++i)
      {
         first=true;
         xA=tomb_x[i-1]; xB=tomb_x[i]; xC=tomb_x[i+1]; xD=tomb_x[i+2];
         yA=tomb_y[i-1]; yB=tomb_y[i]; yC=tomb_y[i+1]; yD=tomb_y[i+2];

/*         a3=(-xA+3*xB-3*xC+xD)/6f;
         a2=(xA-2*xB+xC)/2f;
         a1=(-xA+xC)/2f;
         a0=(xA+4*xB+xC)/6f;

         b3=(-yA+3*yB-3*yC+yD)/6f;
         b2=(yA-2*yB+yC)/2f;
         b1=(-yA+yC)/2f;
         b0=(yA+4*yB+yC)/6f;
*/

         for (j=0; j<=N; ++j)
         {
            t=(float)j/(float)N;
            X=(float)Math.pow((1-t),3)*xA+3*t*(1-t)*(1-t)*xB+3*t*t*(1-t)*xC+t*t*t*xD;
            Y=(float)Math.pow((1-t),3)*yA+3*t*(1-t)*(1-t)*yB+3*t*t*(1-t)*yC+t*t*t*yD;
            if (first)
            {
               first=false;
               X_prev=X; Y_prev=Y;
            }
            else
            {
               g.drawLine(w2v_x(X_prev),w2v_y(Y_prev), w2v_x(X),w2v_y(Y));
               X_prev=X; Y_prev=Y;
            }
         }
      }//end of 'for (i=1; i<=num_points; ++i)'
   }

   void show_lines (Graphics g)
   {
      g.setColor(Color.blue);
      for (int i=1; i<=pos-1; ++i)
         g.drawLine(w2v_x(tomb_x[i]), w2v_y(tomb_y[i]), w2v_x(tomb_x[i+1]), w2v_y(tomb_y[i+1]));
      if (open_or_close==CLOSE) g.drawLine(w2v_x(tomb_x[pos]), w2v_y(tomb_y[pos]), w2v_x(tomb_x[1]), w2v_y(tomb_y[1]));
   }

   void draw_curve() {
      draw_or_undraw=DRAW;
      repaint();
   }

   void undraw_curve() {
      draw_or_undraw=UNDRAW;
      repaint();
   }

   void clear_screen() {
      clear=true;
      pos=0;
      repaint();
   }

   void undo() {
      if (pos>=1) {
         --pos;
         repaint();
      }
   }

   void open_curve() {
      open_or_close=OPEN;
      repaint();
   }

   void close_curve() {
      open_or_close=CLOSE;
      repaint();
   }

   void move_point() {
      move_or_put=MOVE;
   }

   void put_point() {
      move_or_put=PUT;
   }

   void show_lines() {
      show_or_hide_lines=SHOW;
      repaint();
   }

   void hide_lines() {
      show_or_hide_lines=HIDE;
      repaint();
   }

   int w2v_x(float x) {
      return (int)((x-Wx2)*((float)(Vx1-Vx2)/(Wx1-Wx2))+Vx2);
   }
   int w2v_y(float y) {
      return (int)((y-Wy2)*((float)(Vy1-Vy2)/(Wy1-Wy2))+Vy2);
   }
   float v2w_x(int x) {
      return (x-Vx2)*((Wx1-Wx2)/(float)(Vx1-Vx2))+Wx2;
   }
   float v2w_y(int y) {
      return (y-Vy2)*((Wy1-Wy2)/(float)(Vy1-Vy2))+Wy2;
   }
}

class BezierControls extends Panel
{
   BezierCanvas canvas;
   Frame frame;
   Button open_close, show_hide, put_move, draw_undraw, clear, undo, exit;

   public BezierControls(BezierCanvas c, Frame f)
   {
      this.canvas = c;
      this.frame = f;
      add(open_close = new Button("Close curve"));
      add(show_hide = new Button("Show lines"));
      add(put_move = new Button("Move point"));
      add(draw_undraw = new Button("Undraw curve"));
      add(clear = new Button("Clear"));
      add(undo = new Button("Undo"));
      add(exit = new Button("Exit"));
   }

   public boolean handleEvent(Event e)
   {
      boolean handled = false;
      switch (e.id)
      {
         case Event.ACTION_EVENT:
            if (e.arg=="Draw  curve ") {
               draw_undraw.setLabel("Undraw curve");
               canvas.draw_curve();
               handled=true;
            }
            if (e.arg=="Undraw curve") {
               draw_undraw.setLabel("Draw  curve ");
               canvas.undraw_curve();
               handled=true;
            }
            if (e.arg=="Clear") {
               canvas.clear_screen();
               handled=true;
            }
            if (e.arg=="Undo") {
               canvas.undo();
               handled=true;
            }
            if (e.arg=="Exit") {
               frame.dispose();
               System.exit(0);
               handled=true;
            }
            if (e.arg=="Open curve") {
               open_close.setLabel("Close curve");
               canvas.open_curve();
               handled=true;
            }
            if (e.arg=="Close curve") {
               open_close.setLabel("Open curve");
               canvas.close_curve();
               handled=true;
            }
            if (e.arg=="Move point") {
               put_move.setLabel("Put point");
               canvas.move_point();
               handled=true;
            }
            if (e.arg=="Put point") {
               put_move.setLabel("Move point");
               canvas.put_point();
               handled=true;
            }
            if (e.arg=="Show lines") {
               show_hide.setLabel("Hide lines");
               canvas.show_lines();
               handled=true;
            }
            if (e.arg=="Hide lines") {
               show_hide.setLabel("Show lines");
               canvas.hide_lines();
               handled=true;
            }
            break;
      }//end of switch
      return handled;
   }
}

class MyWindow extends Frame
{
   MyWindow (String title) {
      super(title);
   }

   public boolean handleEvent(Event e)
   {
      boolean handled = false;
      switch (e.id)
      {
         case Event.WINDOW_DESTROY:
            dispose();
            System.exit(0);
            handled = true;
            break;
      }
      return handled;
   }
}
