/****************************************************************************/
/*                     Lagrange interpolci megvalositsa                  */
/****************************************************************************/

import java.awt.event.*;
import java.awt.*;
import java.applet.*;

import java.util.Vector;

//---------------------------------------------------------------------
//Interpolacio osztaly
//(Appletkent es programkent is futtathato)
//---------------------------------------------------------------------
public class Interpolation extends Applet{
        DrawPanel panel;                //rajzfelulet
        DrawControls controls;  //gombok
        static boolean app = true;      //appletkent fut-e?

//Applet init
        public void init() {
        setLayout(new BorderLayout());
        panel = new DrawPanel();
        controls = new DrawControls(panel);
        add("Center", panel);
        add("South",controls);
        }

        public void destroy() {
        remove(panel);
        remove(controls);
        }

//Main fuggveny
        public static void main(String args[]) {
        Frame f = new Frame("Interpolcis polinom rajzolsa");
        Interpolation interpolation = new Interpolation();
        interpolation.app = false;
        interpolation.init();
        interpolation.start();

        f.add("Center", interpolation);
        f.setSize(300, 300);
        f.show();
        }

        public String getAppletInfo() {
                return "Interpolci";
        }
}

//---------------------------------------------------------------------
//A rajzfeluletet megvalosito osztaly
//---------------------------------------------------------------------
class DrawPanel extends Panel implements MouseListener {

        public static final int INPUT = 0;                      //pontok megadasara var
        public static final int INTERPOLATION = 1;      //kirajzolja a polinomot
        int     mode = INPUT;   //az aktualis allapot
        Vector points = new Vector();   //a megadott pontok
        Vector calc = new Vector();             //a kalkulalt polinom pontok

//ktor
        public DrawPanel() {
        setBackground(Color.white);
        addMouseListener(this);
        }

//Allapot megvaltoztatasa
        public void setDrawMode(int mode) {
        switch (mode) {
                case INPUT:
                case INTERPOLATION:
                        this.mode = mode;
                        break;
                default:
                        throw new IllegalArgumentException();
        }
        repaint();
        }

//Pontok torlese
        public void clearPoints() {
        /*points.clear();
        calc.clear();
        */
        points = new Vector();
        calc = new Vector();
        }


//Az eger felengedesekor taroljuk a megadott pontot, ha INPUT modban vagyunk
        public void mouseReleased(MouseEvent e) {
                e.consume();
                if ((mode==INPUT) && (points.size()<10)) {      //max. 10 db pont
                        points.addElement(new Point(e.getX(), e.getY()));
                        repaint();
                }
        }

        public void mousePressed(MouseEvent e) {
        }

        public void mouseEntered(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }

        public void mouseClicked(MouseEvent e) {
        }

//Kirajzolas
        public void paint(Graphics g) {
        int np = points.size();

        g.setColor(getForeground());
        for (int i=0; i < np; i++) {    //Megadott pontok kirajzolasa
                Point p = (Point)points.elementAt(i);
                g.drawRect(p.x, p.y, 2, 2);
        }
        if (mode == INTERPOLATION) {    //Ha kell
                g.setColor(Color.black);        //a polinom kirajzolasa
                for (int i=1; i<calc.size(); i++) {
                        Point p1 = (Point)calc.elementAt(i-1);
                        Point p2 = (Point)calc.elementAt(i);
                        g.drawLine(p1.x, p1.y, p2.x, p2.y);
                }
        }
        }

//Polinom ertek szamitasa x pontban (Lagrange interpolacio)
        int calculatex(int x) {
        int i, j, n, xj;
        double sum = 0.0;
        double prod;
        n = points.size();

        for (i=0;i<n;i++) {     //szumma
                Point p = (Point)points.elementAt(i);
                prod=(double)1.0;
                for (j=0;j<n;j++) {     //produktum
                        xj = ((Point)points.elementAt(j)).x;
                        if (p.x != xj) prod*=((double)(x-xj))/(p.x-xj);
                }
                sum+=p.y*prod;
        }
        return (int)Math.round(sum);
        }

//A calc vektor feltoltese a polinom ertekekkel
        public void calculate() {
        if (calc.size() != 0)
                return;
        int i = 0;
        Dimension d = getSize();
        int l = 1;
        while (i<d.width) {
                calc.addElement(new Point (i,calculatex(i)));
                i = i+l;
        }
        }
}

//---------------------------------------------------------------------
//A gombokat tartalmazo panel
//---------------------------------------------------------------------
class DrawControls extends Panel implements ActionListener {
    DrawPanel target;  //hivatkozas a rajzfeluletre

//Ktor
        public DrawControls(DrawPanel target) {
        this.target = target;
        setLayout(new FlowLayout());
        setBackground(Color.lightGray);
        target.setForeground(Color.red);

        Button b; //gombok

        b = new Button("Szmt");
        b.addActionListener(this);
        add(b);
        b = new Button("Trl");
        b.addActionListener(this);
        add(b);
        if (!Interpolation.app) {  //kilep gomb csak ha nem applet
                b = new Button("Kilp");
                b.addActionListener(this);
                add(b);
        }
        }

//Esemenykezelo a gombok miatt
    public void actionPerformed(ActionEvent e) {
        String arg = e.getActionCommand();

        if ("Trl".equals(arg)) {      //ha toroles
                target.clearPoints();
            target.setDrawMode(DrawPanel.INPUT);
        } else if ("Szmt".equals(arg)) {      //ha szamitas
            target.calculate();
                target.setDrawMode(DrawPanel.INTERPOLATION);
        } else if ("Kilp".equals(arg)) {       //ha kilepes
                System.exit(0);
        }
    }
}




